/*
 * Copyright (C) 2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "para.h"
#include "http_recv.cmdline.h"
#include "http.h"

#define BUFSIZE 8192

struct gengetopt_args_info conf;

/* no encryption */
void (*crypt_function_recv)(unsigned long len, const unsigned char *indata, unsigned char *outdata) = NULL;
void (*crypt_function_send)(unsigned long len, const unsigned char *indata, unsigned char *outdata) = NULL;

/*
 * we always log to stderr
 */
void para_log(int ll, char* fmt,...)
{
	va_list argp;

	if (ll < conf.loglevel_arg)
		return;
	va_start(argp, fmt);
	vfprintf(stderr, fmt, argp);
	va_end(argp);
}

static char *make_request_msg(void)
{
	char *ret, *hn = para_hostname();
	ret = make_message("%s\nHost: %s\nUser-Agent: para_http_recv/%s\n\n\n",
		HTTP_GET_MSG, hn, VERSION);
	free(hn);
	return ret;
}

/*
 * MAIN
 */
int main(int argc, char *argv[])
{

	int sockfd, ret;
	ssize_t numbytes = 0;
	struct hostent *he;
	struct sockaddr_in their_addr;
	char *rq, buf[BUFSIZE];

	cmdline_parser(argc, argv, &conf);
	if (conf.loglevel_arg <= NOTICE)
		cmdline_parser_print_version();
	/* get the host info */
	PARA_NOTICE_LOG("getting host info of %s\n", conf.hostname_arg);
	if (!(he = get_host_info(conf.hostname_arg)))
		exit(EXIT_FAILURE);
	/* get new socket */
	if ((sockfd = get_socket()) < 0)
		exit(EXIT_FAILURE);
	/* init their_addr */
	init_sockaddr(&their_addr, conf.server_port_arg, he);
	/* connect */
	PARA_NOTICE_LOG("connecting to %s...\n", conf.hostname_arg);
	if (do_connect(sockfd, &their_addr) < 0) {
		PARA_EMERG_LOG("connect failed (%s)\n", strerror(errno));
		exit(EXIT_FAILURE);
	}
	rq = make_request_msg();
	PARA_NOTICE_LOG("sending request message:\n%s\n", rq);
	ret = send_va_buffer(sockfd, "%s", rq);
	free(rq);
	if (ret < 0)
		exit(EXIT_FAILURE);
	ret = recv_pattern(sockfd, HTTP_OK_MSG, MAXLINE);
	if (ret < 0) {
		PARA_EMERG_LOG("did not receive ok msg (ret = %d)\n", ret);
		exit(EXIT_FAILURE);
	}
	PARA_NOTICE_LOG("%s", "received ok msg, streaming\n");
	/* write server output to stdout */
	while ((numbytes = recv_buffer(sockfd, buf, BUFSIZE)) > 0) {
		ret = write(STDOUT_FILENO, buf, numbytes);
		if (ret < 0)
			exit(EXIT_FAILURE);
		if (ret < numbytes)
			PARA_DEBUG_LOG("short write (%d/%d)\n", ret, numbytes);
	}
	if (numbytes < 0) {
		PARA_ERROR_LOG("read %i (%s)\n", numbytes, strerror(errno));
		exit(EXIT_FAILURE);
	}
	if (!numbytes)
		PARA_NOTICE_LOG("%s", "connection closed by peer\n");
	close(sockfd);
	return 0;
}
